==> fap-0.9.0pre1.tar.bz2: fap-0.9.0pre1/src/audio/input/ttaBackend.h <==
/*
 *  Copyright (C) 2006-2010 Leandro Nini
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef TTA_H
#define TTA_H

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#ifdef BUILD_TTA
#include <stdio.h>
#include "libtta/libtta.h"

#include "inputBackend.h"
#include <fx.h>

/*****************************************************************/

#include "configFrame.h"

class ttaConfig : public configFrame {
	FXDECLARE(ttaConfig)

private:
	ttaConfig() {};
	ttaConfig(const ttaConfig&);
	ttaConfig& operator=(const ttaConfig&);

public:
	ttaConfig(FXPacker* win);
	virtual ~ttaConfig() {};
};

/*****************************************************************/

class ttaBackend : public inputBackend {

private:
	typedef struct {
		tta::TTA_io_callback iocb;
		FXFile *handle;
	} tta_wrapper;

private:
	FXFile		_file;

	tta::tta_decoder	*TTA;
	tta_wrapper	io;
	TTAuint32	_sampleSize;
	TTAuint8	*_decodeBuf;
	TTAuint32	_bufOffset;
	TTAuint32	_bufSize;

private:
	static TTAint32 CALLBACK read_callback(tta::TTA_io_callback *io, TTAuint8 *buffer, TTAuint32 size);
	static TTAint64 CALLBACK seek_callback(tta::TTA_io_callback *io, TTAint64 offset);

	static const char* getError(const tta::TTA_CODEC_STATUS e);

public:
	ttaBackend();
	~ttaBackend();

	static const char name[];

	/// Factory method
	static input* factory() { return new ttaBackend(); }

	/// Check if we support ext
	static bool supports(const char* fileName);

	/// Get supported extension
	FXString ext() const;

	/// Open file
	bool open(const FXString& fileName);

	/// Close file
	void close();

	/// Rewind to start
	const bool rewind();

	/// Get samplerate
	const unsigned int samplerate() const {
		return songLoaded()?
			TTA->info.sps:
			0;
	}

	/// Get channels
	const unsigned int channels() const {
		return songLoaded()?
			TTA->info.nch:
			0;
	}

	/// Get precision
	const sample_t precision() const {
		return songLoaded()?
			(TTA->info.bps==16)?S16:U8:
			S16;
	}

	/// Callback function
	size_t fillBuffer(void* buffer, const size_t bufferSize, const unsigned int seconds);

	/// Gapless support
	const bool gapless() { return true; } //check

	/// Open config dialog
	void config(FXPacker* win) { new ttaConfig(win); }
};

#endif

#endif

==> fap-0.9.2.tar.bz2: fap-0.9.2/src/audio/input/ttaBackend.h <==
/*
 *  Copyright (C) 2006-2011 Leandro Nini
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef TTA_H
#define TTA_H

#ifdef HAVE_CONFIG_H
#  include "config.h"
#endif

#ifdef BUILD_TTA
#include <stdio.h>
#include "libtta/libtta.h"

#include "inputBackend.h"
#include <fx.h>

/*****************************************************************/

#include "configFrame.h"

class ttaConfig : public configFrame {
	FXDECLARE(ttaConfig)

private:
	ttaConfig() {};
	ttaConfig(const ttaConfig&);
	ttaConfig& operator=(const ttaConfig&);

public:
	ttaConfig(FXPacker* win);
	virtual ~ttaConfig() {};
};

/*****************************************************************/

class ttaBackend : public inputBackend {

private:
	typedef struct {
		TTA_io_callback iocb;
		FXFile *handle;
	} tta_wrapper;

private:
	FXFile		_file;

	tta::tta_decoder	*TTA;
	tta_wrapper	io;
	TTAuint32	_sampleSize;
	TTAuint8	*_decodeBuf;
	TTAuint32	_bufOffset;
	TTAuint32	_bufSize;
	TTA_info	_info;

private:
	static TTAint32 CALLBACK read_callback(TTA_io_callback *io, TTAuint8 *buffer, TTAuint32 size);
	static TTAint64 CALLBACK seek_callback(TTA_io_callback *io, TTAint64 offset);

	static const char* getError(const TTA_CODEC_STATUS e);

private:
	ttaBackend();

public:
	~ttaBackend();

	static const char name[];

	/// Factory method
	static input* factory() { return new ttaBackend(); }

	/// Check if we support ext
	static bool supports(const char* fileName);

	/// Get supported extension
	FXString ext() const;

	/// Open file
	bool open(const FXString& fileName);

	/// Close file
	void close();

	/// Rewind to start
	bool rewind();

	/// Get samplerate
	unsigned int samplerate() const {
		return songLoaded()?
			_info.sps:
			0;
	}

	/// Get channels
	unsigned int channels() const {
		return songLoaded()?
			_info.nch:
			0;
	}

	/// Get precision
	sample_t precision() const {
		return songLoaded()?
			(_info.bps==16)?S16:U8:
			S16;
	}

	/// Callback function
	size_t fillBuffer(void* buffer, const size_t bufferSize, const unsigned int seconds);

	/// Gapless support
	bool gapless() const { return true; } //check

	/// Open config dialog
	void config(FXPacker* win) { new ttaConfig(win); }
};

#endif

#endif
